/*
 *   This file was automatically generated by version 1.7 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Fri Nov 28 17:00:03 1997
 */
#ifndef __CEXTRACT__
#if __STDC__

extern double asinh ( double xx );
extern double j0 ( double x );
extern double y0 ( double x );
extern double jn ( int n, double x );
extern double ndtr ( double a );
extern double erfc ( double a );
extern double erf ( double x );
extern double acosh ( double x );
extern double atanh ( double x );
extern double j1 ( double x );
extern double y1 ( double x );
extern int mtherr ( char *name, int code );
extern double polevl ( double x, double coef[], int N );
extern double p1evl ( double x, double coef[], int N );
extern double yn ( int n, double x );

#else /* __STDC__ */

extern double asinh (/* double xx */);
extern double j0 (/* double x */);
extern double y0 (/* double x */);
extern double jn (/* int n, double x */);
extern double ndtr (/* double a */);
extern double erfc (/* double a */);
extern double erf (/* double x */);
extern double acosh (/* double x */);
extern double atanh (/* double x */);
extern double j1 (/* double x */);
extern double y1 (/* double x */);
extern int mtherr (/* char *name, int code */);
extern double polevl (/* double x, double coef[], int N */);
extern double p1evl (/* double x, double coef[], int N */);
extern double yn (/* int n, double x */);

#endif /* __STDC__ */
#endif /* __CEXTRACT__ */
